#include "SettingsCategory.hpp"
#include "Setting.hpp"
#include "Settings.hpp"
#include <QtCore/QJsonObject>

using namespace std;
using namespace MellowPlayer::Domain;

SettingsCategory::SettingsCategory(const SettingsCategory::Data& categoryData, Settings* appSettings) : QObject(appSettings), data_(categoryData)
{
    for (int i = 0; i < data_.parameters.count(); ++i) {
        QJsonObject parameterObj = data_.parameters.at(i).toObject();
        Setting::Data settingData;
        settingData.name = parameterObj.value("name").toString();
        settingData.toolTip = parameterObj.value("tooltip").toString();
        settingData.type = parameterObj.value("type").toString();
        settingData.key = parameterObj.value("key").toString();
        settingData.defaultValue = parameterObj.value("default").toVariant();
        settingData.enableCondition = parameterObj.value("enabled").toString();
        settings_.append(new Setting(*appSettings, *this, settingData));
    }
}

void SettingsCategory::resolveDependencies()
{
    for (Setting* setting : settings_)
        setting->resolveDependency();
}

const QString& SettingsCategory::name() const
{
    return data_.name;
}

const QString& SettingsCategory::icon() const
{
    return data_.icon;
}

const QString& SettingsCategory::key() const
{
    return data_.key;
}

const QList<Setting*>& SettingsCategory::toList() const
{
    return settings_;
}

Setting& SettingsCategory::get(const QString& key) const
{
    for (Setting* param : settings_)
        if (param->key() == key)
            return *param;
    throw runtime_error("Unknown setting: " + data_.key.toStdString() + "/" + key.toStdString());
}

void SettingsCategory::restoreDefaults()
{
    for (Setting* setting : settings_)
        setting->restoreDefaults();
}
